package sheet6;

/* This class implements a multiplicative linear
congruence generator (LCG)
*
* Methods:
* long longRand() for pseudo-random number z_i
*
* @param m: module
* @param a: factor
* @param z: pseudo-random number (resp. grain)
*
* @author: F. Timmermann, 17.06.2010
* @author: O. Stenzel, 03.07.2012
*/
public class lcgb {

	/* Initialization of parameters */
	private long z, m = 2147483647, a = 16807;
	/* Constructs a LCG with module 2147483647, factor
	16807 and grain 1 */
	public lcgb(int z) {
	this.z = z;
	}

	/* Recursive calculation of numbers z in {0,...,m-1}
	under mod(m)-rule
	*
	* @return pseudo-random number in {0,...,m-1}
	*/
	public long longRand() {
		z = (a * z) % m;
		return z;
	}
	
	/* Transformation of the numbers in {0,...,m-1} in
	interval [0,1)
	*
	* @ return z/m pseudo-random number u in [0,1)
	(uniformly distributed??)
	*/
	public double uniRand() {
		double z = longRand();
		return z / m;
	}
	
	public static boolean PearsonChiSquaredTest(int n, int R, int z) {
		
		/* Initialization with fixed parameters */
		lcgb mylcgb = new lcgb(z);
		double[] freq = new double[R];
		/* Simulation of <n> pseudo-random numbers
		and frequency count */
		for (int i = 0; i < n; i++) {
		int idx = (int) Math.floor(mylcgb.uniRand()*R);
		freq[idx] += 1;
		}
		/* Calculation of test statistic */
		double t = mylcgb.tValue(freq, n, R);
		double quantile = 16.92;
		/* Null-hypothesis rejected or not rejected? */
		boolean rejected = false;
		if (t > quantile) {
			System.out.println("\n" + "Null-hypothesis on [0,1]-uniform distribution rejected, T = " + t 
			+ " quantile: 16.92");
		    rejected = true;
		} else {
			System.out.println("\n" + "Null-hypothesis on [0,1]-uniform distribution not rejected, T = " + t
			+ "	quantile: 16.92");
		}
		return rejected;
	}
	
	
	/* Calculation of the test statistic
	*
	* @ return t ( = test statistic)
	*/
	public double tValue(double[] freq, int n, int R) {
		double n_d = (double) n;
		double R_d = (double) R;
		double t = 0;
		for (int j = 0; j < R; j++) {
			t += (freq[j] - n_d / R_d) * (freq[j] - n_d /
			R_d) / (n_d / R_d);
		} return t;
	}

		
	
/* Main program */
static public void main(String[] args) {
	
	boolean rejected = PearsonChiSquaredTest(100000, 10, 1);
	
	int rejectedCount = 0;
	for(int i = 0; i < 100; i++) {
		if(PearsonChiSquaredTest(100000, 10, 100 * i)) {
			rejectedCount++;
		}
	}
	System.out.println(rejectedCount + " of 100 simulations rejected");
}
}